package com.javahonk.report.util;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.mail.internet.MimeMessage;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.MailException;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.mail.javamail.MimeMessagePreparator;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import freemarker.cache.FileTemplateLoader;
import freemarker.template.Configuration;

public class MailHelper {
	
	private static final Logger logger = LogManager.getLogger(MailHelper.class.getName());
	
	@Autowired
	private MailSender mailSender;
	
	@Autowired
	@Qualifier("mailSender")
	private JavaMailSender javaMailSender;
	
	@Autowired
	Configuration freemarkerConfiguration;
	
	@Value("${from_address}")
	private String fromAddress;
	
	@Value("${subject}")
	private String subject;
	
	public void sendTemplatedMimeMessage(
			final Map<String, Object> templatedMimeMessage,
			final String template,
			final List<String> toAddresses,
			final Map<String, File> attachments) throws IOException{
		
		FileTemplateLoader templateLoader = null;
		templateLoader = new FileTemplateLoader(new File("src/main/resources"));
		freemarkerConfiguration.setTemplateLoader(templateLoader);		
		
		MimeMessagePreparator preparator;
		try {
			preparator = new MimeMessagePreparator() {
				
				String messageText = 
						FreeMarkerTemplateUtils.processTemplateIntoString(
								freemarkerConfiguration.getTemplate(template),templatedMimeMessage);
				
				@Override
				public void prepare(MimeMessage message) throws Exception {
					
					MimeMessageHelper helper = new MimeMessageHelper(message, true);
					
					for(String address : toAddresses){
						helper.addTo(address);
					}
					
					helper.setFrom(fromAddress);
					helper.setSubject(subject);
					
					if(attachments!=null){
						
						for(Entry<String, File> attachment : attachments.entrySet()){
							
							helper.addAttachment(attachment.getKey(), attachment.getValue());
						}
						
					}
					
					helper.setText(messageText,true);
				}
			};
			
			this.javaMailSender.send(preparator);
			
		} catch (Exception ex) {
			
			logger.error("Failed to send email for templated message:" +
					"\nFrom:" + fromAddress +
					"\nTo:" + toAddresses +
					"\nMessageMap:" + templatedMimeMessage, ex);
			
		} 
		
	}
	
	public void sendMimeMessage(
			final String mimeMessage,
			final String fromAddress,
			final List<String> toAddresses,
			final String mailSubject,
			final Map<String, File> attachments){
		
		MimeMessagePreparator preparator = new MimeMessagePreparator() {
			
			@Override
			public void prepare(MimeMessage message) throws Exception {
				
				MimeMessageHelper helper = new MimeMessageHelper(message, true);
				
				for(String address : toAddresses){
					helper.addTo(address);
				}
				
				helper.setFrom(fromAddress);
				helper.setSubject(mailSubject);
				
				if(attachments!=null){
					
					for(Entry<String, File> attachment : attachments.entrySet()){
						
						helper.addAttachment(attachment.getKey(), attachment.getValue());
					}
					
				}
				
				helper.setText(mimeMessage);
			}
		};
		
		try{
			
            this.javaMailSender.send(preparator);
            
        }
        catch (MailException ex) {
        	
        	logger.error("Failed to send email for templated message:" +
					"\nFrom:" + fromAddress +
					"\nTo:" + toAddresses +
					"\nMessage:" + mimeMessage, ex);
        }
		
	}

	public void sendSimpleMailMessage(
			SimpleMailMessage templateMessage,
			String message){
		
		SimpleMailMessage msg = new SimpleMailMessage(templateMessage);
		
		msg.setText(message);
		
		try{
			
            this.mailSender.send(msg);
            
        }
        catch (MailException ex) {
        	
			logger.error("Failed to send email for templated message:" +
					"\nFrom:" + templateMessage.getFrom() +
					"\nTo:" + templateMessage.getTo() +
					"\nMessage:" + templateMessage.getText(), ex);

        }
		
	}
	
}
